VERSION 8000; ;8300

* -----------------------------------------------------------------------------
* @filedoc
* @file Bwk_Leitungen_Generieren.Csx
* @brief Versorgungsleitungen fr Topografielinien generieren 
* 
* Dieses Beispielskript generiert Versorgungsleitungen parallel zu vorhandenen Topografielinien. 
* Fr die Generierung werden Topografielinien mit Kode 401 der Schicht "Topografie" ausgewertet. 
* Die Hhenposition der Leitung wird im Abstand von 1,5 m zur DGM-Oberflche ermittelt.
*
* -----------------------------------------------------------------------------
* Copyright (C) IB&T
* 
* Gewhrleistung und Nutzung
* 
* Die Nutzung des Skriptes erfolgt stets auf eigene Gefahr. IB&T bernimmt keine
* Gewhr dafr, dass die Nutzung dieses Skriptes zu Ihrem gewnschten Ergebnis
* fhrt. Da CardScripte vorhandene Projektdaten durch Direktzugriff auf die
* jeweiligen Datenbanken auch ohne weitere Sicherheitsabfragen unbemerkt verndern
* oder lschen knnen, erfordert eine Nutzung entsprechende Kenntnisse und groe
* Sorgfalt. IB&T haftet nicht fr eventuell entstehende Datenverluste und
* Folgeschden, die durch die Nutzung von CardScripten entstehen.
*
* -----------------------------------------------------------------------------
* @author TB
* @endfiledoc
* -----------------------------------------------------------------------------
* @history
* -----------------------------------------------------------------------------

BILDSCHIRM GRAFIK                   | gibt die Meldungen im Arbeitsprotokoll aus

* --- Zuweisungen -----------------------------------------------------------

!SchichtName = "Bestand Topografie"         | Name der Schicht mit Topografielinien
KodeA = 401                         | Kodebereich Anfang
KodeE = 401                         | Kodebereich Ende
DiffOb = 1,5                        | Abstand zur DGM-Oberflche
Rrad = 0,3                          | Wandhhe der Leitungen
Rwan = 0,1                          | Wanddicke der Leitungen

* --- Deklarationen ---------------------------------------------------------

SYMBOLE /TOPOSYS/ TopoSys                   | Symbol fr das System der Topografiedaten
SYMBOLE /TOPOLINE/ Linie                    | Symbol fr eine Topografielinie
SYMBOLE /TOPOLINEINDEX/ XLinie              | Symbol fr das Linienindexobjekt 

SYMBOLE /DTMSYS/ DgmSys                     | Symbol fr das System der DGM-Daten
SYMBOLE /DTM/ Dgm                           | Symbol fr das DGM

SYMBOLE /BUILDSYS/ BauwerkSys               | Symbol fr das System der Bauwerke
SYMBOLE /BUILDSTRUCTURE/ Bauwerk            | Symbol fr das Bauwerk (Versorgungsleitungen)
SYMBOLE /BUILDASSEMBLY/ Baugruppe           | Symbol fr die Baugruppe (Rohrsystem)
SYMBOLE /BUILDMODULE/ Bauteil               | Symbol fr das Bauteil (Einzelrohr)
SYMBOLE /BUILDBODY/ Koerper                 | Symbol fr den Baukrper (Teil des Einzelrohrs)
SYMBOLE /BUILDPOINTARRAY/ Array1; Array2    | Symbole fr Hilfspolygone

SYMBOLE /COORDINATE/ Ko1; Ko2               | Symbole fr Hilfspunkte
SYMBOLE /LINE/ HLinie; CLinie1; CLinie2     | Symbole fr Hilfslinien

* --- Ausfhrungen ----------------------------------------------------------

* Objekte fr die Datensysteme
TopoSys = Project.GetTopoSys()              | liefert das Objekt fr das System der Topografiedaten
DgmSys = Project.GetDtmSys()                | liefert das Objekt fr das System der DGM-Daten
BauwerkSys = Project.GetBuildSys()          | liefert das Objekt fr das System der Bauwerke

* DGM-Objekt
!DgmName = !DGM_WAHL(0)
Dgm = DgmSys.GetDtm(!DgmName)

* Linienindexobjekt 
XLinie = TopoSys.GetLineIndex(!SchichtName)      | liefert alle Linien einer Schicht 
XLinie.SetCodeRange(KodeA; KodeE)                | setzt den auszuwertenden Kodebereich

* ... Schleife ber kodierte Linien einer Schicht

* Linienobjekt
SOLANGE Linie = XLinie.GetNext()                 | liefert die nchste Linie

  LinNr = Linie.GetNumber()                      | liefert die Indexnummer der Linie 
  EleAnz = Linie.GetVertexCount()-1              | liefert die Anzahl der Elemente der Linie 
  
  * 1. Ebene: Bauwerksobjekt ("Leitung")
  TEXTGEN !LName; "Leitung_#"; LinNr             | Name der Leitung (aus Liniennummer)
  Bauwerk = BauwerkSys.GetStructure(!LName)      | liefert ein vorhandenes Bauwerk
  WENN Bauwerk <> $NULL DANN                     | Wenn Bauwerk vorhanden...
     Bauwerk.Delete()                            | lscht das Bauwerk mit allen Baugruppen
  WEITER
  Bauwerk = BauwerkSys.CreateStructure(!LName)   | erzeugt das Bauwerk ("Leitung")
 
  * 2. Ebene: Baugruppenobjekt ("Rohrsystem")
  TEXTGEN !GrName; "Rohre_#"; LinNr              | Name des Rohrsystems (aus Liniennummer)
  Baugruppe= Bauwerk.CreateAssembly(!GrName)     | erzeugt Baugruppe fr jeweiliges Rohrsystem

* ... Schleife ber alle Elemente der Linie

  SCHLEIFE i = 1; EleAnz 

    * 3. Ebene: Bauteilobjekt ("Einzelrohr")
    TEXTGEN !Bauteil; "Rohr_#"; i                | Name des jeweiligen Einzelrohrs (aus Elementnummer)
    Bauteil = Baugruppe.CreateModule(!Bauteil)   | erzeugt Bauteil fr jeweiliges Einzelrohr 
    Bauteil.SetMaterial("neutral_blau2(#16)")    | setzt Material des Einzelrohrs 

    * 4. Ebene: Baukrperobjekt (Teil des "Einzelrohrs")
    Koerper = Bauteil.CreateBody()               | erzeugt Baukrper des Bauteils

    * Hilfspunktobjekte "Ko1" und "Ko2": Koordinate und Hhe Rohranfang und -ende aus vorh. Linienelement 
    Ko1 = Linie.GetCoordinate(i)                 | erzeugt Hilfspunkt Rohranfang: 
    X1 = Ko1.GetX()                              |   Rechtswert 
    Y1 = Ko1.GetY()                              |   Hochwert 
    Z1 = Dgm.GetHeight(X1; Y1)-DiffOb            |   Hhe aus: Gelndehhe - Differenz
    Ko2 = Linie.GetCoordinate(i+1)               | erzeugt Hilfspunkt Rohrende: 
    X2 = Ko2.GetX()                              |   Rechtswert 
    Y2 = Ko2.GetY()                              |   Hochwert 
    Z2 = Dgm.GetHeight(X2; Y2)-DiffOb            |   Hhe aus: Gelndehhe - Differenz
    Ko1.Draw(2; 9)                               | visualaisiert Koordinate Rohranfang 

    * Hilfslinienobjekt "HLinie": Rohrelement aus Rohranfang ("Ko1") und -ende ("Ko2")
    HLINIE = NEWOBJECT /LINE/                    | erzeugt Hilfslinie:
    HLinie.SetCoordinate(Ko1)                    |   Anfangspunkt des Elements
    HLinie.SetCoordinate2(Ko2)                   |   Endpunkt des Elements
    EleLae = Hlinie.GetDistance()                | liefert die Rohrlnge (fr Station Rohrende)
    HLinie.Draw()                                | visualisiert Rohrelement

    * Hilfslinienobjekt "CLinie1" und "CLinie2": senkrechte Linien am Rohranfang und -ende
    CLinie1 = HLinie.GetCrossLine(0)             | liefert senkrechte Linie am Anfang der Linie "HLinie" (= Rohranfang)
    CLinie2 = HLinie.GetCrossLine(EleLae)        | liefert senkrechte Linie am Ende der Linie "HLinie" (= Rohrende)

    * Hilfspolygonobjekt: Rohranfang (= Kreispolygon)
    Array1 = Koerper.CreatePointArray(1)                 | erzeugt Anfangspolygon (1 = geschlossen):
    Array1.CreateRing(Rrad; Rwan; 32)                    |   erzeugt Kreispolygon mit Innen- und Auenkreis (32 = Anzahl der Kreispunkte)
    Array1.SetPositionLine(CLinie1; Z1)                  |   platziert das Kreispolygon mithilfe der Linie "CLinie1"
    Array1.SetIncline(0,0; 1,0)                          |   stellt das Kreispolygon senkrecht auf, um ein waagerechtes Rohr zu erhalten
    
    * Hilfspolygonobjekt: Rohrende (= Kreispolygon)      
    Array2 = Koerper.CreatePointArray(1)                 | erzeugt Endpolygon (s.o.)
    Array2.CreateRing(Rrad; Rwan; 32)
    Array2.SetPositionLine(CLinie2; Z2)
    Array2.SetIncline(0,0; 1,0)
    
    * verbindet Anfangs- und Endpolygon
    Koerper.CreateTube(Array1; Array2)

    *  generiert fr erstes / letztes Rohrpolygon eine Abschlussflche zwischen Innen- und Auenkreis
    WENN i = 1 DANN      Koerper.CreateCover(Array1; 0; "neutral_wei(#1)")
    WENN i = EleAnz DANN Koerper.CreateCover(Array2; 0; "neutral_wei(#1)")
    
* ... Ende der Schleife ber alle Elemente der Linie
  ENDE SCHLEIFE
  Baugruppe = $NULL

* ... Ende der Schleife ber alle Linien eines Kodes einer Schicht
ENDE SOLANGE 

MELDUNG NEUTRAL "Versorgungsleitungen wurden generiert"